/*
    Copyright (c) 2011-2012 Andrey Sibiryov <me@kobology.ru>
    Copyright (c) 2011-2012 Other contributors as noted in the AUTHORS file.

    This file is part of Cocaine.

    Cocaine is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    Cocaine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>. 
*/

#ifndef COCAINE_LOGGING_HPP
#define COCAINE_LOGGING_HPP

#include "cocaine/common.hpp"

#include "cocaine/api/logger.hpp"

#include "cocaine/helpers/format.hpp"

#define COCAINE_LOG(l, v, ...) \
    if(l->verbosity() >= v) l->emit(v, __VA_ARGS__);

#define COCAINE_LOG_DEBUG(l, ...) \
    COCAINE_LOG(l, logging::debug, __VA_ARGS__)

#define COCAINE_LOG_INFO(l, ...) \
    COCAINE_LOG(l, logging::info, __VA_ARGS__)

#define COCAINE_LOG_WARNING(l, ...) \
    COCAINE_LOG(l, logging::warning, __VA_ARGS__)

#define COCAINE_LOG_ERROR(l, ...) \
    COCAINE_LOG(l, logging::error, __VA_ARGS__)

namespace cocaine { namespace logging {

struct log_t {
    log_t(context_t& context,
          const std::string& source):
        m_logger(context.logger()),
        m_source(source)
    { }

    logging::priorities
    verbosity() const {
        return m_logger.verbosity();
    }

    template<typename... Args>
    void
    emit(logging::priorities level,
         const std::string& format,
         const Args&... args)
    {
        m_logger.emit(level, m_source, cocaine::format(format, args...));
    }

    void
    emit(logging::priorities level,
         const std::string& message)
    {
        m_logger.emit(level, m_source, message);
    }

private:
    api::logger_t& m_logger;

    // The name of this log, to be used as the logging source.
    const std::string m_source;
};

}}

#endif
